@@ -8,6 +8,7 @@ from collections import namedtuple
 from itertools import chain
 
 from metaflow.metaflow_environment import MetaflowEnvironment
+from metaflow.current import current
 from metaflow.exception import (
     MetaflowNotFound,
     MetaflowNamespaceMismatch,
@@ -17,7 +18,7 @@ from metaflow.exception import (
 from metaflow.metaflow_config import DEFAULT_METADATA, MAX_ATTEMPTS
 from metaflow.plugins import ENVIRONMENTS, METADATA_PROVIDERS
 from metaflow.unbounded_foreach import CONTROL_TASK_TAG
-from metaflow.util import cached_property, resolve_identity, to_unicode
+from metaflow.util import cached_property, resolve_identity, to_unicode, is_stringish
 
 from .filecache import FileCache
 
@@ -47,6 +48,11 @@ def metadata(ms):
     for example, not allow access to information stored in remote
     metadata providers.
 
+    Note that you don't typically have to call this function directly. Usually
+    the metadata provider is set through the Metaflow configuration file. If you
+    need to switch between multiple providers, you can use the `METAFLOW_PROFILE`
+    environment variable to switch between configurations.
+
     Parameters
     ----------
     ms : string
@@ -58,7 +64,7 @@ def metadata(ms):
     -------
     string
         The description of the metadata selected (equivalent to the result of
-        get_metadata())
+        get_metadata()).
     """
     global current_metadata
     infos = ms.split("@", 1)
@@ -90,11 +96,12 @@ def get_metadata():
     """
     Returns the current Metadata provider.
 
-    This call returns the current Metadata being used to return information
-    about Metaflow objects.
+    If this is not set explicitly using `metadata`, the default value is
+    determined through the Metaflow configuration. You can use this call to
+    check that your configuration is set up properly.
 
-    If this is not set explicitly using metadata(), the default value is
-    determined through environment variables.
+    If multiple configuration profiles are present, this call returns the one
+    selected through the `METAFLOW_PROFILE` environment variable.
 
     Returns
     -------
@@ -110,10 +117,8 @@ def get_metadata():
 
 def default_metadata():
     """
-    Resets the Metadata provider to the default value.
-
-    The default value of the Metadata provider is determined through a combination of
-    environment variables.
+    Resets the Metadata provider to the default value, that is, to the value
+    that was used prior to any `metadata` calls.
 
     Returns
     -------
@@ -121,6 +126,12 @@ def default_metadata():
         The result of get_metadata() after resetting the provider.
     """
     global current_metadata
+
+    # We first check if we are in a flow -- if that is the case, we use the
+    # metadata provider that is being used there
+    if current._metadata_str:
+        return metadata(current._metadata_str)
+
     default = [m for m in METADATA_PROVIDERS if m.TYPE == DEFAULT_METADATA]
     if default:
         current_metadata = default[0]
@@ -174,15 +185,13 @@ def get_namespace():
 
 def default_namespace():
     """
-    Sets or resets the namespace used to filter objects.
-
-    The default namespace is in the form 'user:<username>' and is intended to filter
-    objects belonging to the user.
+    Resets the namespace used to filter objects to the default one, i.e. the one that was
+    used prior to any `namespace` calls.
 
     Returns
     -------
     string
-        The result of get_namespace() after
+        The result of get_namespace() after the namespace has been reset.
     """
     global current_namespace
     current_namespace = resolve_identity()
@@ -198,11 +207,10 @@ class Metaflow(object):
 
     Attributes
     ----------
-    flows : List of all flows.
-        Returns the list of all flows. Note that only flows present in the set namespace will
-        be returned. A flow is present in a namespace if it has at least one run in the
-        namespace.
-
+    flows : List[Flow]
+        Returns the list of all `Flow` objects known to this metadata provider. Note that only
+        flows present in the current namespace will be returned. A `Flow` is present in a namespace
+        if it has at least one run in the namespace.
     """
 
     def __init__(self):
@@ -302,7 +310,11 @@ class MetaflowObject(object):
     Attributes
     ----------
     tags : Set
-        Tags associated with the object.
+        Tags associated with the run this object belongs to (user and system tags).
+    user_tags: Set
+        User tags associated with the run this object belongs to.
+    system_tags: Set
+        System tags associated with the run this object belongs to.
     created_at : datetime
         Date and time this object was first created.
     parent : MetaflowObject
@@ -379,6 +391,8 @@ class MetaflowObject(object):
         self._tags = frozenset(
             chain(self._object.get("system_tags") or [], self._object.get("tags") or [])
         )
+        self._user_tags = frozenset(self._object.get("tags") or [])
+        self._system_tags = frozenset(self._object.get("system_tags") or [])
 
         if _namespace_check and not self.is_in_namespace():
             raise MetaflowNamespaceMismatch(current_namespace)
@@ -545,6 +559,30 @@ class MetaflowObject(object):
         """
         return self._tags
 
+    @property
+    def system_tags(self):
+        """
+        System defined tags associated with this object.
+
+        Returns
+        -------
+        Set[string]
+            System tags associated with the object
+        """
+        return self._system_tags
+
+    @property
+    def user_tags(self):
+        """
+        User defined tags associated with this object.
+
+        Returns
+        -------
+        Set[string]
+            User tags associated with the object
+        """
+        return self._user_tags
+
     @property
     def created_at(self):
         """
@@ -664,6 +702,33 @@ class MetaflowObject(object):
 
 
 class MetaflowData(object):
+    """
+    Container of data artifacts produced by a `Task`. This object is
+    instantiated through `Task.data`.
+
+    `MetaflowData` allows results to be retrieved by their name
+    through a convenient dot notation:
+
+    ```python
+    Task(...).data.my_object
+    ```
+
+    You can also test the existence of an object
+
+    ```python
+    if 'my_object' in Task(...).data:
+        print('my_object found')
+    ```
+
+    Note that this container relies on the local cache to load all data
+    artifacts. If your `Task` contains a lot of data, a more efficient
+    approach is to load artifacts individually like so
+
+    ```
+    Task(...)['my_object'].data
+    ```
+    """
+
     def __init__(self, artifacts):
         self._artifacts = dict((art.id, art) for art in artifacts)
 
@@ -682,22 +747,31 @@ class MetaflowData(object):
 
 class MetaflowCode(object):
     """
-    Describes the code that is occasionally stored with a run.
+    Snapshot of the code used to execute this `Run`. Instantiate the object through
+    `Run(...).code` (if all steps are executed remotely) or `Task(...).code` for an
+    individual task. The code package is the same for all steps of a `Run`.
 
-    A code package will contain the version of Metaflow that was used (all the files comprising
-    the Metaflow library) as well as selected files from the directory containing the Python
-    file of the FlowSpec.
+    `MetaflowCode` includes a package of the user-defined `FlowSpec` class and supporting
+    files, as well as a snapshot of the Metaflow library itself.
+
+    Currently `MetaflowCode` objects are stored only for `Run`s that have at least one `Step`
+    executing outside the user's local environment.
+
+    You can extract code in the directory `snapshot` like so:
+    ```
+    Run(...).code.extractall(path='snapshot')
+    ```
 
     Attributes
     ----------
     path : string
-        Location (in the datastore provider) of the code package
+        Location (in the datastore provider) of the code package.
     info : Dict
-        Dictionary of information related to this code-package
+        Dictionary of information related to this code-package.
     flowspec : string
-        Source code of the file containing the FlowSpec in this code package
+        Source code of the file containing the `FlowSpec` in this code package.
     tarball : TarFile
-        Tar ball containing all the code
+        Python standard library `tarfile.TarFile` archive containing all the code.
     """
 
     def __init__(self, flow_name, code_package):
@@ -775,16 +849,19 @@ class MetaflowCode(object):
 
 class DataArtifact(MetaflowObject):
     """
-    A single data artifact and associated metadata.
+    A single data artifact and associated metadata. Note that this object does
+    not contain other objects as it is the leaf object in the hierarchy.
 
     Attributes
     ----------
     data : object
-        The unpickled representation of the data contained in this artifact
+        The data contained in this artifact, that is, the object produced during
+        execution of this run.
     sha : string
-        Encoding representing the unique identity of this artifact
+        A unique ID of this artifact.
     finished_at : datetime
-        Alias for created_at
+        Corresponds roughly to the `Task.finished_at` time of the parent `Task`.
+        An alias for `DataArtifact.created_at`.
     """
 
     _NAME = "artifact"
@@ -895,48 +972,53 @@ class DataArtifact(MetaflowObject):
 
 class Task(MetaflowObject):
     """
-    A Task represents an execution of a step.
+    A `Task` represents an execution of a `Step`.
 
-    As such, it contains all data artifacts associated with that execution as
-    well as all metadata associated with the execution.
+    It contains all `DataArtifact` objects produced by the task as
+    well as metadata related to execution.
 
-    Note that you can also get information about a specific *attempt* of a
-    task. By default, the latest finished attempt is returned but you can
+    Note that the `@retry` decorator may cause multiple attempts of
+    the task to be present. Usually you want the latest attempt, which
+    is what instantiating a `Task` object returns by default. If
+    you need to e.g. retrieve logs from a failed attempt, you can
     explicitly get information about a specific attempt by using the
     following syntax when creating a task:
-    `Task('flow/run/step/task', attempt=<attempt>)`. Note that you will not be able to
-    access a specific attempt of a task through the `.tasks` method of a step
-    for example (that will always return the latest attempt).
+
+    `Task('flow/run/step/task', attempt=<attempt>)`
+
+    where `attempt=0` corresponds to the first attempt etc.
 
     Attributes
     ----------
     metadata : List[Metadata]
-        List of all metadata associated with the task
+        List of all metadata events associated with the task.
     metadata_dict : Dict
-        Dictionary where the keys are the names of the metadata and the value are the values
-        associated with those names
+        A condensed version of `metadata`: A dictionary where keys
+        are names of metadata events and values the latest corresponding event.
     data : MetaflowData
-        Container of all data artifacts produced by this task
+        Container of all data artifacts produced by this task. Note that this
+        call downloads all data locally, so it can be slower than accessing
+        artifacts individually. See `MetaflowData` for more information.
     artifacts : MetaflowArtifacts
-        Container of DataArtifact objects produced by this task
+        Container of `DataArtifact` objects produced by this task.
     successful : boolean
-        True if the task successfully completed
+        True if the task completed successfully.
     finished : boolean
-        True if the task completed
+        True if the task completed.
     exception : object
-        Exception raised by this task if there was one
+        Exception raised by this task if there was one.
     finished_at : datetime
-        Time this task finished
+        Time this task finished.
     runtime_name : string
-        Runtime this task was executed on
+        Runtime this task was executed on.
     stdout : string
-        Standard output for the task execution
+        Standard output for the task execution.
     stderr : string
-        Standard error output for the task execution
+        Standard error output for the task execution.
     code : MetaflowCode
-        Code package for this task (if present)
+        Code package for this task (if present). See `MetaflowCode`.
     environment_info : Dict
-        Information about the execution environment (for example Conda)
+        Information about the execution environment.
     """
 
     _NAME = "task"
@@ -967,16 +1049,59 @@ class Task(MetaflowObject):
             self._NAME, "metadata", None, self._attempt, *self.path_components
         )
         all_metadata = all_metadata if all_metadata else []
-        return [
-            Metadata(
-                name=obj.get("field_name"),
-                value=obj.get("value"),
-                created_at=obj.get("ts_epoch"),
-                type=obj.get("type"),
-                task=self,
+
+        # For "clones" (ie: they have an origin-run-id AND a origin-task-id), we
+        # copy a set of metadata from the original task. This is needed to make things
+        # like logs work (which rely on having proper values for ds-root for example)
+        origin_run_id = None
+        origin_task_id = None
+        result = []
+        existing_keys = []
+        for obj in all_metadata:
+            result.append(
+                Metadata(
+                    name=obj.get("field_name"),
+                    value=obj.get("value"),
+                    created_at=obj.get("ts_epoch"),
+                    type=obj.get("type"),
+                    task=self,
+                )
             )
-            for obj in all_metadata
-        ]
+            existing_keys.append(obj.get("field_name"))
+            if obj.get("field_name") == "origin-run-id":
+                origin_run_id = obj.get("value")
+            elif obj.get("field_name") == "origin-task-id":
+                origin_task_id = obj.get("value")
+
+        if origin_task_id:
+            # This is a "cloned" task. We consider that it has the same
+            # metadata as the last attempt of the cloned task.
+
+            origin_obj_pathcomponents = self.path_components
+            origin_obj_pathcomponents[1] = origin_run_id
+            origin_obj_pathcomponents[3] = origin_task_id
+            origin_task = Task(
+                "/".join(origin_obj_pathcomponents), _namespace_check=False
+            )
+            latest_metadata = {
+                m.name: m
+                for m in sorted(origin_task.metadata, key=lambda m: m.created_at)
+            }
+            # We point to ourselves in the Metadata object
+            for v in latest_metadata.values():
+                if v.name in existing_keys:
+                    continue
+                result.append(
+                    Metadata(
+                        name=v.name,
+                        value=v.value,
+                        created_at=v.created_at,
+                        type=v.type,
+                        task=self,
+                    )
+                )
+
+        return result
 
     @property
     def metadata_dict(self):
@@ -1280,35 +1405,52 @@ class Task(MetaflowObject):
         if not env_type:
             return None
         env = [m for m in ENVIRONMENTS + [MetaflowEnvironment] if m.TYPE == env_type][0]
-        return env.get_client_info(self.path_components[0], self.metadata_dict)
+        meta_dict = self.metadata_dict
+        return env.get_client_info(self.path_components[0], meta_dict)
 
     def _load_log(self, stream):
-        log_location = self.metadata_dict.get("log_location_%s" % stream)
+        meta_dict = self.metadata_dict
+        log_location = meta_dict.get("log_location_%s" % stream)
         if log_location:
             return self._load_log_legacy(log_location, stream)
         else:
-            return "".join(line + "\n" for _, line in self.loglines(stream))
+            return "".join(
+                line + "\n" for _, line in self.loglines(stream, meta_dict=meta_dict)
+            )
 
     def _get_logsize(self, stream):
-        log_location = self.metadata_dict.get("log_location_%s" % stream)
+        meta_dict = self.metadata_dict
+        log_location = meta_dict.get("log_location_%s" % stream)
         if log_location:
             return self._legacy_log_size(log_location, stream)
         else:
-            return self._log_size(stream)
+            return self._log_size(stream, meta_dict)
 
-    def loglines(self, stream, as_unicode=True):
+    def loglines(self, stream, as_unicode=True, meta_dict=None):
         """
         Return an iterator over (utc_timestamp, logline) tuples.
 
-        If as_unicode=False, logline is returned as a byte object. Otherwise,
-        it is returned as a (unicode) string.
+        Parameters
+        ----------
+        stream : string
+            Either 'stdout' or 'stderr'.
+        as_unicode : boolean
+            If as_unicode=False, each logline is returned as a byte object. Otherwise,
+            it is returned as a (unicode) string.
+
+        Returns
+        -------
+        Iterator[(datetime, string)]
+            Iterator over timestamp, logline pairs.
         """
         from metaflow.mflog.mflog import merge_logs
 
         global filecache
 
-        ds_type = self.metadata_dict.get("ds-type")
-        ds_root = self.metadata_dict.get("ds-root")
+        if meta_dict is None:
+            meta_dict = self.metadata_dict
+        ds_type = meta_dict.get("ds-type")
+        ds_root = meta_dict.get("ds-root")
         if ds_type is None or ds_root is None:
             yield None, ""
             return
@@ -1355,11 +1497,11 @@ class Task(MetaflowObject):
             ds_type, location, logtype, int(attempt), *self.path_components
         )
 
-    def _log_size(self, stream):
+    def _log_size(self, stream, meta_dict):
         global filecache
 
-        ds_type = self.metadata_dict.get("ds-type")
-        ds_root = self.metadata_dict.get("ds-root")
+        ds_type = meta_dict.get("ds-type")
+        ds_root = meta_dict.get("ds-root")
         if ds_type is None or ds_root is None:
             return 0
         if filecache is None:
@@ -1373,20 +1515,21 @@ class Task(MetaflowObject):
 
 class Step(MetaflowObject):
     """
-    A Step represents a user-defined Step (a method annotated with the @step decorator).
+    A `Step` represents a user-defined step, that is, a method annotated with the `@step` decorator.
 
-    As such, it contains all Tasks associated with the step (ie: all executions of the
-    Step). A linear Step will have only one associated task whereas a foreach Step will have
-    multiple Tasks.
+    It contains `Task` objects associated with the step, that is, all executions of the
+    `Step`. The step may contain multiple `Task`s in the case of a foreach step.
 
     Attributes
     ----------
     task : Task
-        Returns a Task object from the step
+        The first `Task` object in this step. This is a shortcut for retrieving the only
+        task contained in a non-foreach step.
     finished_at : datetime
-        Time this step finished (time of completion of the last task)
+        Time when the latest `Task` of this step finished. Note that in the case of foreaches,
+        this time may change during execution of the step.
     environment_info : Dict
-        Information about the execution environment (for example Conda)
+        Information about the execution environment.
     """
 
     _NAME = "step"
@@ -1410,41 +1553,45 @@ class Step(MetaflowObject):
 
     def tasks(self, *tags):
         """
-        Returns an iterator over all the tasks in the step.
+        [Legacy function - do not use]
 
-        An optional filter is available that allows you to filter on tags.
-        If tags are specified, only tasks associated with all specified tags
-        are returned.
+        Returns an iterator over all `Task` objects in the step. This is an alias
+        to iterating the object itself, i.e.
+        ```
+        list(Step(...)) == list(Step(...).tasks())
+        ```
 
         Parameters
         ----------
         tags : string
-            Tags to match
+            No op (legacy functionality)
 
         Returns
         -------
         Iterator[Task]
-            Iterator over Task objects in this step
+            Iterator over all `Task` objects in this step.
         """
         return self._filtered_children(*tags)
 
     @property
     def control_task(self):
         """
+        [Unpublished API - use with caution!]
+
         Returns a Control Task object belonging to this step.
         This is useful when the step only contains one control task.
+
         Returns
         -------
         Task
             A control task in the step
         """
-        children = super(Step, self).__iter__()
-        for t in children:
-            if CONTROL_TASK_TAG in t.tags:
-                return t
+        return next(self.control_tasks(), None)
 
     def control_tasks(self, *tags):
         """
+        [Unpublished API - use with caution!]
+
         Returns an iterator over all the control tasks in the step.
         An optional filter is available that allows you to filter on tags. The
         control tasks returned if the filter is specified will contain all the
@@ -1459,11 +1606,23 @@ class Step(MetaflowObject):
             Iterator over Control Task objects in this step
         """
         children = super(Step, self).__iter__()
-        filter_tags = [CONTROL_TASK_TAG]
-        filter_tags.extend(tags)
         for child in children:
-            if all(tag in child.tags for tag in filter_tags):
+            # first filter by standard tag filters
+            if not all(tag in child.tags for tag in tags):
+                continue
+            # Then look for control task indicator in one of two ways
+            # Look in tags - this path will activate for metadata service
+            # backends that pre-date tag mutation release
+            if CONTROL_TASK_TAG in child.tags:
                 yield child
+            else:
+                # Look in task metadata
+                for task_metadata in child.metadata:
+                    if (
+                        task_metadata.name == "internal_task_type"
+                        and task_metadata.value == CONTROL_TASK_TAG
+                    ):
+                        yield child
 
     def __iter__(self):
         children = super(Step, self).__iter__()
@@ -1511,24 +1670,22 @@ class Step(MetaflowObject):
 
 class Run(MetaflowObject):
     """
-    A Run represents an execution of a Flow
-
-    As such, it contains all Steps associated with the flow.
+    A `Run` represents an execution of a `Flow`. It is a container of `Step`s.
 
     Attributes
     ----------
     data : MetaflowData
-        Container of all data artifacts produced by this run
+        a shortcut to run['end'].task.data, i.e. data produced by this run.
     successful : boolean
-        True if the run successfully completed
+        True if the run completed successfully.
     finished : boolean
-        True if the run completed
+        True if the run completed.
     finished_at : datetime
-        Time this run finished
+        Time this run finished.
     code : MetaflowCode
-        Code package for this run (if present)
+        Code package for this run (if present). See `MetaflowCode`.
     end_task : Task
-        Task for the end step (if it is present already)
+        `Task` for the end step (if it is present already).
     """
 
     _NAME = "run"
@@ -1541,21 +1698,23 @@ class Run(MetaflowObject):
 
     def steps(self, *tags):
         """
-        Returns an iterator over all the steps in the run.
+        [Legacy function - do not use]
 
-        An optional filter is available that allows you to filter on tags.
-        If tags are specified, only steps associated with all specified tags
-        are returned.
+        Returns an iterator over all `Step` objects in the step. This is an alias
+        to iterating the object itself, i.e.
+        ```
+        list(Run(...)) == list(Run(...).steps())
+        ```
 
         Parameters
         ----------
         tags : string
-            Tags to match
+            No op (legacy functionality)
 
         Returns
         -------
         Iterator[Step]
-            Iterator over Step objects in this run
+            Iterator over `Step` objects in this run.
         """
         return self._filtered_children(*tags)
 
@@ -1667,20 +1826,134 @@ class Run(MetaflowObject):
 
         return end_step.task
 
+    def add_tag(self, tag):
+        """
+        Add a tag to this `Run`.
+
+        Note that if the tag is already a system tag, it is not added as a user tag,
+        and no error is thrown.
+
+        Parameters
+        ----------
+        tag : string
+            Tag to add.
+        """
+
+        # For backwards compatibility with Netflix's early version of this functionality,
+        # this function shall accept both an individual tag AND iterables of tags.
+        #
+        # Iterable of tags support shall be removed in future once existing
+        # usage has been migrated off.
+        if is_stringish(tag):
+            tag = [tag]
+        return self.replace_tag([], tag)
+
+    def add_tags(self, tags):
+        """
+        Add one or more tags to this `Run`.
+
+        Note that if any tag is already a system tag, it is not added as a user tag
+        and no error is thrown.
+
+        Parameters
+        ----------
+        tags : Iterable[string]
+            Tags to add.
+        """
+        return self.replace_tag([], tags)
+
+    def remove_tag(self, tag):
+        """
+        Remove one tag from this `Run`.
+
+        Removing a system tag is an error. Removing a non-existent
+        user tag is a no-op.
+
+        Parameters
+        ----------
+        tag : string
+            Tag to remove.
+        """
+
+        # For backwards compatibility with Netflix's early version of this functionality,
+        # this function shall accept both an individual tag AND iterables of tags.
+        #
+        # Iterable of tags support shall be removed in future once existing
+        # usage has been migrated off.
+        if is_stringish(tag):
+            tag = [tag]
+        return self.replace_tag(tag, [])
+
+    def remove_tags(self, tags):
+        """
+        Remove one or more tags to this `Run`.
+
+        Removing a system tag will result in an error. Removing a non-existent
+        user tag is a no-op.
+
+        Parameters
+        ----------
+        tags : Iterable[string]
+            Tags to remove.
+        """
+        return self.replace_tags(tags, [])
+
+    def replace_tag(self, tag_to_remove, tag_to_add):
+        """
+        Remove a tag and add a tag atomically. Removal is done first.
+        The rules for `Run.add_tag` and `Run.remove_tag` also apply here.
+
+        Parameters
+        ----------
+        tag_to_remove : string
+            Tag to remove.
+        tag_to_add : string
+            Tag to add.
+        """
+
+        # For backwards compatibility with Netflix's early version of this functionality,
+        # this function shall accept both individual tags AND iterables of tags.
+        #
+        # Iterable of tags support shall be removed in future once existing
+        # usage has been migrated off.
+        if is_stringish(tag_to_remove):
+            tag_to_remove = [tag_to_remove]
+        if is_stringish(tag_to_add):
+            tag_to_add = [tag_to_add]
+        return self.replace_tags(tag_to_remove, tag_to_add)
+
+    def replace_tags(self, tags_to_remove, tags_to_add):
+        """
+        Remove and add tags atomically; the removal is done first.
+        The rules for `Run.add_tag` and `Run.remove_tag` also apply here.
+
+        Parameters
+        ----------
+        tags_to_remove : Iterable[string]
+            Tags to remove.
+        tags_to_add : Iterable[string]
+            Tags to add.
+        """
+        flow_id = self.path_components[0]
+        final_user_tags = self._metaflow.metadata.mutate_user_tags_for_run(
+            flow_id, self.id, tags_to_remove=tags_to_remove, tags_to_add=tags_to_add
+        )
+        # refresh Run object with the latest tags
+        self._user_tags = frozenset(final_user_tags)
+        self._tags = frozenset([*self._user_tags, *self._system_tags])
+
 
 class Flow(MetaflowObject):
     """
     A Flow represents all existing flows with a certain name, in other words,
-    classes derived from 'FlowSpec'
-
-    As such, it contains all Runs (executions of a flow) related to this flow.
+    classes derived from `FlowSpec`. A container of `Run` objects.
 
     Attributes
     ----------
     latest_run : Run
-        Latest Run (in progress or completed, successfully or not) of this Flow
+        Latest `Run` (in progress or completed, successfully or not) of this flow.
     latest_successful_run : Run
-        Latest successfully completed Run of this Flow
+        Latest successfully completed `Run` of this flow.
     """
 
     _NAME = "flow"
@@ -1722,21 +1995,21 @@ class Flow(MetaflowObject):
 
     def runs(self, *tags):
         """
-        Returns an iterator over all the runs in the flow.
+        Returns an iterator over all `Run`s of this flow.
 
         An optional filter is available that allows you to filter on tags.
-        If tags are specified, only runs associated with all specified tags
-        are returned.
+        If multiple tags are specified, only runs that have all the
+        specified tags are returned.
 
         Parameters
         ----------
         tags : string
-            Tags to match
+            Tags to match.
 
         Returns
         -------
         Iterator[Run]
-            Iterator over Run objects in this flow
+            Iterator over `Run` objects in this flow.
         """
         return self._filtered_children(*tags)
 
