@@ -3,9 +3,12 @@ from __future__ import print_function
 import json
 import time
 import math
+import re
 import sys
 import os
 import traceback
+from collections import namedtuple
+from functools import partial
 from hashlib import sha1
 from tempfile import NamedTemporaryFile
 from multiprocessing import Process, Queue
@@ -22,11 +25,13 @@ except:
     from urllib.parse import urlparse
     from queue import Full as QueueFull
 
-import click
+if __name__ == "__main__":
+    # When launched standalone, point to our parent metaflow
+    sys.path.insert(
+        0, os.path.abspath(os.path.join(os.path.dirname(__file__), "../../"))
+    )
 
-# s3op can be launched as a stand-alone script. We must set
-# PYTHONPATH for the parent Metaflow explicitly.
-sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "../../")))
+from metaflow._vendor import click
 
 # we use Metaflow's parallel_imap_unordered instead of
 # multiprocessing.Pool because https://bugs.python.org/issue31886
@@ -39,6 +44,10 @@ NUM_WORKERS_DEFAULT = 64
 DOWNLOAD_FILE_THRESHOLD = 2 * TransferConfig().multipart_threshold
 DOWNLOAD_MAX_CHUNK = 2 * 1024 * 1024 * 1024 - 1
 
+RANGE_MATCH = re.compile(r"bytes (?P<start>[0-9]+)-(?P<end>[0-9]+)/(?P<total>[0-9]+)")
+
+S3Config = namedtuple("S3Config", "role session_vars client_params")
+
 
 class S3Url(object):
     def __init__(
@@ -75,10 +84,11 @@ ERROR_URL_ACCESS_DENIED = 7
 ERROR_WORKER_EXCEPTION = 8
 ERROR_VERIFY_FAILED = 9
 ERROR_LOCAL_FILE_NOT_FOUND = 10
+ERROR_INVALID_RANGE = 11
 
 
 def format_triplet(prefix, url="", local=""):
-    return u" ".join(url_quote(x).decode("utf-8") for x in (prefix, url, local))
+    return " ".join(url_quote(x).decode("utf-8") for x in (prefix, url, local))
 
 
 # I can't understand what's the right way to deal
@@ -93,13 +103,15 @@ def normalize_client_error(err):
             return 403
         if error_code == "NoSuchKey":
             return 404
+        if error_code == "InvalidRange":
+            return 416
     return error_code
 
 
 # S3 worker pool
 
 
-def worker(result_file_name, queue, mode):
+def worker(result_file_name, queue, mode, s3config):
     # Interpret mode, it can either be a single op or something like
     # info_download or info_upload which implies:
     #  - for download: we need to return the information as well
@@ -128,6 +140,8 @@ def worker(result_file_name, queue, mode):
                 to_return = {"error": ERROR_URL_NOT_FOUND, "raise_error": err}
             elif error_code == 403:
                 to_return = {"error": ERROR_URL_ACCESS_DENIED, "raise_error": err}
+            elif error_code == 416:
+                to_return = {"error": ERROR_INVALID_RANGE, "raise_error": err}
             else:
                 to_return = {"error": error_code, "raise_error": err}
         return to_return
@@ -136,7 +150,11 @@ def worker(result_file_name, queue, mode):
         try:
             from metaflow.datatools.s3util import get_s3_client
 
-            s3, client_error = get_s3_client()
+            s3, client_error = get_s3_client(
+                s3_role_arn=s3config.role,
+                s3_session_vars=s3config.session_vars,
+                s3_client_params=s3config.client_params,
+            )
             while True:
                 url, idx = queue.get()
                 if url is None:
@@ -155,9 +173,23 @@ def worker(result_file_name, queue, mode):
                             resp = s3.get_object(
                                 Bucket=url.bucket, Key=url.path, Range=url.range
                             )
+                            range_result = resp["ContentRange"]
+                            range_result_match = RANGE_MATCH.match(range_result)
+                            if range_result_match is None:
+                                raise RuntimeError(
+                                    "Wrong format for ContentRange: %s"
+                                    % str(range_result)
+                                )
+                            range_result = {
+                                x: int(range_result_match.group(x))
+                                for x in ["total", "start", "end"]
+                            }
                         else:
                             resp = s3.get_object(Bucket=url.bucket, Key=url.path)
+                            range_result = None
                         sz = resp["ContentLength"]
+                        if range_result is None:
+                            range_result = {"total": sz, "start": 0, "end": sz - 1}
                         if not url.range and sz > DOWNLOAD_FILE_THRESHOLD:
                             # In this case, it is more efficient to use download_file as it
                             # will download multiple parts in parallel (it does it after
@@ -186,7 +218,12 @@ def worker(result_file_name, queue, mode):
                     if pre_op_info:
 
                         with open("%s_meta" % url.local, mode="w") as f:
-                            args = {"size": resp["ContentLength"]}
+                            # Get range information
+
+                            args = {
+                                "size": resp["ContentLength"],
+                                "range_result": range_result,
+                            }
                             if resp["ContentType"]:
                                 args["content_type"] = resp["ContentType"]
                             if resp["Metadata"] is not None:
@@ -228,7 +265,7 @@ def worker(result_file_name, queue, mode):
             sys.exit(ERROR_WORKER_EXCEPTION)
 
 
-def start_workers(mode, urls, num_workers):
+def start_workers(mode, urls, num_workers, s3config):
     # We start the minimum of len(urls) or num_workers to avoid starting
     # workers that will definitely do nothing
     num_workers = min(num_workers, len(urls))
@@ -250,7 +287,10 @@ def start_workers(mode, urls, num_workers):
     with TempDir() as output_dir:
         for i in range(num_workers):
             file_path = os.path.join(output_dir, str(i))
-            p = Process(target=worker, args=(file_path, queue, mode))
+            p = Process(
+                target=worker,
+                args=(file_path, queue, mode, s3config),
+            )
             p.start()
             procs[p] = file_path
 
@@ -276,13 +316,13 @@ def start_workers(mode, urls, num_workers):
     return sz_results
 
 
-def process_urls(mode, urls, verbose, num_workers):
+def process_urls(mode, urls, verbose, num_workers, s3config):
 
     if verbose:
         print("%sing %d files.." % (mode.capitalize(), len(urls)), file=sys.stderr)
 
     start = time.time()
-    sz_results = start_workers(mode, urls, num_workers)
+    sz_results = start_workers(mode, urls, num_workers, s3config)
     end = time.time()
 
     if verbose:
@@ -320,15 +360,20 @@ def with_unit(x):
 # required by @aws_retry decorator, which needs the reset_client
 # method. Otherwise they would be just stand-alone functions.
 class S3Ops(object):
-    def __init__(self):
+    def __init__(self, s3config):
         self.s3 = None
+        self.s3config = s3config
         self.client_error = None
 
     def reset_client(self, hard_reset=False):
         from metaflow.datatools.s3util import get_s3_client
 
         if hard_reset or self.s3 is None:
-            self.s3, self.client_error = get_s3_client()
+            self.s3, self.client_error = get_s3_client(
+                s3_role_arn=self.s3config.role,
+                s3_session_vars=self.s3config.session_vars,
+                s3_client_params=self.s3config.client_params,
+            )
 
     @aws_retry
     def get_info(self, url):
@@ -412,18 +457,18 @@ class S3Ops(object):
 # This is accomplished by op_ functions below.
 
 
-def op_get_info(urls):
-    s3 = S3Ops()
+def op_get_info(s3config, urls):
+    s3 = S3Ops(s3config)
     return [s3.get_info(url) for url in urls]
 
 
-def op_list_prefix(prefix_urls):
-    s3 = S3Ops()
+def op_list_prefix(s3config, prefix_urls):
+    s3 = S3Ops(s3config)
     return [s3.list_prefix(prefix) for prefix in prefix_urls]
 
 
-def op_list_prefix_nonrecursive(prefix_urls):
-    s3 = S3Ops()
+def op_list_prefix_nonrecursive(s3config, prefix_urls):
+    s3 = S3Ops(s3config)
     return [s3.list_prefix(prefix, delimiter="/") for prefix in prefix_urls]
 
 
@@ -467,17 +512,24 @@ def verify_results(urls, verbose=False):
                 exit(ERROR_VERIFY_FAILED, url)
 
 
-def generate_local_path(url, suffix=None):
+def generate_local_path(url, range="whole", suffix=None):
     # this function generates a safe local file name corresponding to
     # an S3 URL. URLs may be longer than maximum file length limit on Linux,
     # so we mostly hash the URL but retain the leaf part as a convenience
     # feature to ease eyeballing
+    # We also call out "range" specifically to allow multiple ranges for the same
+    # file to be downloaded in parallel.
+    if range is None:
+        range = "whole"
+    if range != "whole":
+        # It will be of the form `bytes=%d-` or `bytes=-%d` or `bytes=%d-%d`
+        range = range[6:].replace("-", "_")
     quoted = url_quote(url)
     fname = quoted.split(b"/")[-1].replace(b".", b"_").replace(b"-", b"_")
     sha = sha1(quoted).hexdigest()
     if suffix:
-        return u"-".join((sha, fname.decode("utf-8"), suffix))
-    return u"-".join((sha, fname.decode("utf-8")))
+        return "-".join((sha, fname.decode("utf-8"), range, suffix))
+    return "-".join((sha, fname.decode("utf-8"), range))
 
 
 def parallel_op(op, lst, num_workers):
@@ -535,8 +587,43 @@ def cli():
     show_default=True,
     help="Download prefixes recursively.",
 )
+@click.option(
+    "--s3role",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Role to assume when getting the S3 client",
+)
+@click.option(
+    "--s3sessionvars",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Session vars to set when getting the S3 client",
+)
+@click.option(
+    "--s3clientparams",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Client parameters to set when getting the S3 client",
+)
 @click.argument("prefixes", nargs=-1)
-def lst(prefixes, inputs=None, num_workers=None, recursive=None):
+def lst(
+    prefixes,
+    inputs=None,
+    num_workers=None,
+    recursive=None,
+    s3role=None,
+    s3sessionvars=None,
+    s3clientparams=None,
+):
+
+    s3config = S3Config(
+        s3role,
+        json.loads(s3sessionvars) if s3sessionvars else None,
+        json.loads(s3clientparams) if s3clientparams else None,
+    )
 
     urllist = []
     for prefix, _ in _populate_prefixes(prefixes, inputs):
@@ -552,7 +639,11 @@ def lst(prefixes, inputs=None, num_workers=None, recursive=None):
             exit(ERROR_INVALID_URL, url)
         urllist.append(url)
 
-    op = op_list_prefix if recursive else op_list_prefix_nonrecursive
+    op = (
+        partial(op_list_prefix, s3config)
+        if recursive
+        else partial(op_list_prefix_nonrecursive, s3config)
+    )
     urls = []
     for success, prefix_url, ret in parallel_op(op, urllist, num_workers):
         if success:
@@ -604,6 +695,27 @@ def lst(prefixes, inputs=None, num_workers=None, recursive=None):
     show_default=True,
     help="Print S3 URLs upload to on stdout.",
 )
+@click.option(
+    "--s3role",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Role to assume when getting the S3 client",
+)
+@click.option(
+    "--s3sessionvars",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Session vars to set when getting the S3 client",
+)
+@click.option(
+    "--s3clientparams",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Client parameters to set when getting the S3 client",
+)
 def put(
     files=None,
     filelist=None,
@@ -611,6 +723,9 @@ def put(
     verbose=None,
     overwrite=True,
     listing=None,
+    s3role=None,
+    s3sessionvars=None,
+    s3clientparams=None,
 ):
     def _files():
         for local, url in files:
@@ -643,11 +758,17 @@ def put(
             exit(ERROR_NOT_FULL_PATH, url)
         return url
 
+    s3config = S3Config(
+        s3role,
+        json.loads(s3sessionvars) if s3sessionvars else None,
+        json.loads(s3clientparams) if s3clientparams else None,
+    )
+
     urls = list(starmap(_make_url, _files()))
     ul_op = "upload"
     if not overwrite:
         ul_op = "info_upload"
-    sz_results = process_urls(ul_op, urls, verbose, num_workers)
+    sz_results = process_urls(ul_op, urls, verbose, num_workers, s3config)
     urls = [url for url, sz in zip(urls, sz_results) if sz is not None]
     if listing:
         for url in urls:
@@ -722,6 +843,27 @@ def _populate_prefixes(prefixes, inputs):
     show_default=True,
     help="Print S3 URL -> local file mapping on stdout.",
 )
+@click.option(
+    "--s3role",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Role to assume when getting the S3 client",
+)
+@click.option(
+    "--s3sessionvars",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Session vars to set when getting the S3 client",
+)
+@click.option(
+    "--s3clientparams",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Client parameters to set when getting the S3 client",
+)
 @click.argument("prefixes", nargs=-1)
 def get(
     prefixes,
@@ -733,8 +875,17 @@ def get(
     allow_missing=None,
     verbose=None,
     listing=None,
+    s3role=None,
+    s3sessionvars=None,
+    s3clientparams=None,
 ):
 
+    s3config = S3Config(
+        s3role,
+        json.loads(s3sessionvars) if s3sessionvars else None,
+        json.loads(s3clientparams) if s3clientparams else None,
+    )
+
     # Construct a list of URL (prefix) objects
     urllist = []
     for prefix, r in _populate_prefixes(prefixes, inputs):
@@ -743,7 +894,7 @@ def get(
             url=prefix,
             bucket=src.netloc,
             path=src.path.lstrip("/"),
-            local=generate_local_path(prefix),
+            local=generate_local_path(prefix, range=r),
             prefix=prefix,
             range=r,
         )
@@ -756,7 +907,7 @@ def get(
     op = None
     dl_op = "download"
     if recursive:
-        op = op_list_prefix
+        op = partial(op_list_prefix, s3config)
     if verify or verbose or info:
         dl_op = "info_download"
     if op:
@@ -778,7 +929,7 @@ def get(
 
     # exclude the non-existent files from loading
     to_load = [url for url, size in urls if size is not None]
-    sz_results = process_urls(dl_op, to_load, verbose, num_workers)
+    sz_results = process_urls(dl_op, to_load, verbose, num_workers, s3config)
     # We check if there is any access denied
     is_denied = [sz == -ERROR_URL_ACCESS_DENIED for sz in sz_results]
     if any(is_denied):
@@ -844,8 +995,44 @@ def get(
     show_default=True,
     help="Print S3 URL -> local file mapping on stdout.",
 )
+@click.option(
+    "--s3role",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Role to assume when getting the S3 client",
+)
+@click.option(
+    "--s3sessionvars",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Session vars to set when getting the S3 client",
+)
+@click.option(
+    "--s3clientparams",
+    default=None,
+    show_default=True,
+    required=False,
+    help="Client parameters to set when getting the S3 client",
+)
 @click.argument("prefixes", nargs=-1)
-def info(prefixes, num_workers=None, inputs=None, verbose=None, listing=None):
+def info(
+    prefixes,
+    num_workers=None,
+    inputs=None,
+    verbose=None,
+    listing=None,
+    s3role=None,
+    s3sessionvars=None,
+    s3clientparams=None,
+):
+
+    s3config = S3Config(
+        s3role,
+        json.loads(s3sessionvars) if s3sessionvars else None,
+        json.loads(s3clientparams) if s3clientparams else None,
+    )
 
     # Construct a list of URL (prefix) objects
     urllist = []
@@ -863,7 +1050,7 @@ def info(prefixes, num_workers=None, inputs=None, verbose=None, listing=None):
             exit(ERROR_INVALID_URL, url)
         urllist.append(url)
 
-    process_urls("info", urllist, verbose, num_workers)
+    process_urls("info", urllist, verbose, num_workers, s3config)
 
     if listing:
         for url in urllist:
