@@ -3,7 +3,7 @@ import requests
 import json
 
 from threading import Thread
-from metaflow.sidecar_messages import MessageTypes, Message
+from metaflow.sidecar import MessageTypes, Message
 from metaflow.metaflow_config import METADATA_SERVICE_HEADERS
 from metaflow.exception import MetaflowException
 
@@ -20,7 +20,7 @@ class HeartBeatException(MetaflowException):
 class MetadataHeartBeat(object):
     def __init__(self):
         self.headers = METADATA_SERVICE_HEADERS
-        self.req_thread = Thread(target=self.ping)
+        self.req_thread = Thread(target=self._ping)
         self.req_thread.daemon = True
         self.default_frequency_secs = 10
         self.hb_url = None
@@ -28,19 +28,22 @@ class MetadataHeartBeat(object):
     def process_message(self, msg):
         # type: (Message) -> None
         if msg.msg_type == MessageTypes.SHUTDOWN:
-            # todo shutdown doesnt do anything yet? should it still be called
-            self.shutdown()
-        if (not self.req_thread.is_alive()) and msg.msg_type == MessageTypes.LOG_EVENT:
+            self._shutdown()
+        if not self.req_thread.is_alive():
             # set post url
             self.hb_url = msg.payload[HB_URL_KEY]
             # start thread
             self.req_thread.start()
 
-    def ping(self):
+    @classmethod
+    def get_worker(cls):
+        return cls
+
+    def _ping(self):
         retry_counter = 0
         while True:
             try:
-                frequency_secs = self.heartbeat()
+                frequency_secs = self._heartbeat()
 
                 if frequency_secs is None or frequency_secs <= 0:
                     frequency_secs = self.default_frequency_secs
@@ -51,7 +54,7 @@ class MetadataHeartBeat(object):
                 retry_counter = retry_counter + 1
                 time.sleep(4 ** retry_counter)
 
-    def heartbeat(self):
+    def _heartbeat(self):
         if self.hb_url is not None:
             response = requests.post(url=self.hb_url, data="{}", headers=self.headers)
             # Unfortunately, response.json() returns a string that we need
@@ -67,6 +70,6 @@ class MetadataHeartBeat(object):
                 )
         return None
 
-    def shutdown(self):
+    def _shutdown(self):
         # attempts sending one last heartbeat
-        self.heartbeat()
+        self._heartbeat()
