@@ -1,10 +1,10 @@
-import os
 import json
 import logging
-import pkg_resources
+import os
 import sys
 import types
 
+import pkg_resources
 
 from metaflow.exception import MetaflowException
 
@@ -60,42 +60,7 @@ DATASTORE_LOCAL_DIR = ".metaflow"
 DATASTORE_SYSROOT_LOCAL = from_conf("METAFLOW_DATASTORE_SYSROOT_LOCAL")
 # S3 bucket and prefix to store artifacts for 's3' datastore.
 DATASTORE_SYSROOT_S3 = from_conf("METAFLOW_DATASTORE_SYSROOT_S3")
-# S3 datatools root location
-DATATOOLS_SUFFIX = from_conf("METAFLOW_DATATOOLS_SUFFIX", "data")
-DATATOOLS_S3ROOT = from_conf(
-    "METAFLOW_DATATOOLS_S3ROOT",
-    os.path.join(from_conf("METAFLOW_DATASTORE_SYSROOT_S3"), DATATOOLS_SUFFIX)
-    if from_conf("METAFLOW_DATASTORE_SYSROOT_S3")
-    else None,
-)
-# Local datatools root location
-DATATOOLS_LOCALROOT = from_conf(
-    "METAFLOW_DATATOOLS_LOCALROOT",
-    os.path.join(from_conf("METAFLOW_DATASTORE_SYSROOT_LOCAL"), DATATOOLS_SUFFIX)
-    if from_conf("METAFLOW_DATASTORE_SYSROOT_LOCAL")
-    else None,
-)
 
-# Cards related config variables
-DATASTORE_CARD_SUFFIX = "mf.cards"
-DATASTORE_CARD_LOCALROOT = from_conf("METAFLOW_CARD_LOCALROOT")
-DATASTORE_CARD_S3ROOT = from_conf(
-    "METAFLOW_CARD_S3ROOT",
-    os.path.join(from_conf("METAFLOW_DATASTORE_SYSROOT_S3"), DATASTORE_CARD_SUFFIX)
-    if from_conf("METAFLOW_DATASTORE_SYSROOT_S3")
-    else None,
-)
-CARD_NO_WARNING = from_conf("METAFLOW_CARD_NO_WARNING", False)
-
-# S3 endpoint url
-S3_ENDPOINT_URL = from_conf("METAFLOW_S3_ENDPOINT_URL", None)
-S3_VERIFY_CERTIFICATE = from_conf("METAFLOW_S3_VERIFY_CERTIFICATE", None)
-
-# S3 retry configuration
-# This is useful if you want to "fail fast" on S3 operations; use with caution
-# though as this may increase failures. Note that this is the number of *retries*
-# so setting it to 0 means each operation will be tried once.
-S3_RETRY_COUNT = int(from_conf("METAFLOW_S3_RETRY_COUNT", 7))
 
 ###
 # Datastore local cache
@@ -116,6 +81,79 @@ CLIENT_CACHE_MAX_TASKDATASTORE_COUNT = int(
 )
 
 
+###
+# Datatools (S3) configuration
+###
+# S3 endpoint url
+S3_ENDPOINT_URL = from_conf("METAFLOW_S3_ENDPOINT_URL", None)
+S3_VERIFY_CERTIFICATE = from_conf("METAFLOW_S3_VERIFY_CERTIFICATE", None)
+
+# S3 retry configuration
+# This is useful if you want to "fail fast" on S3 operations; use with caution
+# though as this may increase failures. Note that this is the number of *retries*
+# so setting it to 0 means each operation will be tried once.
+S3_RETRY_COUNT = int(from_conf("METAFLOW_S3_RETRY_COUNT", 7))
+
+# Threshold to start printing warnings for an AWS retry
+RETRY_WARNING_THRESHOLD = 3
+
+# S3 datatools root location
+DATATOOLS_SUFFIX = from_conf("METAFLOW_DATATOOLS_SUFFIX", "data")
+DATATOOLS_S3ROOT = from_conf(
+    "METAFLOW_DATATOOLS_S3ROOT",
+    os.path.join(from_conf("METAFLOW_DATASTORE_SYSROOT_S3"), DATATOOLS_SUFFIX)
+    if from_conf("METAFLOW_DATASTORE_SYSROOT_S3")
+    else None,
+)
+
+DATATOOLS_DEFAULT_CLIENT_PARAMS = from_conf(
+    "METAFLOW_DATATOOLS_CLIENT_PARAMS",
+    "{}",
+)
+try:
+    DATATOOLS_DEFAULT_CLIENT_PARAMS = json.loads(DATATOOLS_DEFAULT_CLIENT_PARAMS)
+    if S3_ENDPOINT_URL:
+        DATATOOLS_DEFAULT_CLIENT_PARAMS["endpoint_url"] = S3_ENDPOINT_URL
+    if S3_VERIFY_CERTIFICATE:
+        DATATOOLS_DEFAULT_CLIENT_PARAMS["verify"] = S3_VERIFY_CERTIFICATE
+except json.JSONDecodeError:
+    raise ValueError(
+        "Value for METAFLOW_DATATOOLS_CLIENT_PARAMS is not valid JSON: %s"
+        % DATATOOLS_DEFAULT_CLIENT_PARAMS
+    )
+
+DATATOOLS_DEFAULT_SESSION_VARS = from_conf("METAFLOW_DATATOOLS_SESSION_VARS", "{}")
+try:
+    DATATOOLS_DEFAULT_SESSION_VARS = json.loads(DATATOOLS_DEFAULT_SESSION_VARS)
+except json.JSONDecodeError:
+    raise ValueError(
+        "Value for METAFLOW_DATATOOLS_SESSION_VARS is not valid JSON: %s"
+        % DATATOOLS_DEFAULT_SESSION_VARS
+    )
+
+# Local datatools root location
+DATATOOLS_LOCALROOT = from_conf(
+    "METAFLOW_DATATOOLS_LOCALROOT",
+    os.path.join(from_conf("METAFLOW_DATASTORE_SYSROOT_LOCAL"), DATATOOLS_SUFFIX)
+    if from_conf("METAFLOW_DATASTORE_SYSROOT_LOCAL")
+    else None,
+)
+
+# The root directory to save artifact pulls in, when using S3
+ARTIFACT_LOCALROOT = from_conf("METAFLOW_ARTIFACT_LOCALROOT", os.getcwd())
+
+# Cards related config variables
+DATASTORE_CARD_SUFFIX = "mf.cards"
+DATASTORE_CARD_LOCALROOT = from_conf("METAFLOW_CARD_LOCALROOT")
+DATASTORE_CARD_S3ROOT = from_conf(
+    "METAFLOW_CARD_S3ROOT",
+    os.path.join(from_conf("METAFLOW_DATASTORE_SYSROOT_S3"), DATASTORE_CARD_SUFFIX)
+    if from_conf("METAFLOW_DATASTORE_SYSROOT_S3")
+    else None,
+)
+CARD_NO_WARNING = from_conf("METAFLOW_CARD_NO_WARNING", False)
+
+
 ###
 # Metadata configuration
 ###
@@ -125,6 +163,8 @@ METADATA_SERVICE_AUTH_KEY = from_conf("METAFLOW_SERVICE_AUTH_KEY")
 METADATA_SERVICE_HEADERS = json.loads(from_conf("METAFLOW_SERVICE_HEADERS", "{}"))
 if METADATA_SERVICE_AUTH_KEY is not None:
     METADATA_SERVICE_HEADERS["x-api-key"] = METADATA_SERVICE_AUTH_KEY
+# Checks version compatibility with Metadata service
+METADATA_SERVICE_VERSION_CHECK = from_conf("METAFLOW_SERVICE_VERSION_CHECK", True)
 
 # Default container image
 DEFAULT_CONTAINER_IMAGE = from_conf("METAFLOW_DEFAULT_CONTAINER_IMAGE")
@@ -185,8 +225,13 @@ SFN_EXECUTION_LOG_GROUP_ARN = from_conf("METAFLOW_SFN_EXECUTION_LOG_GROUP_ARN")
 ###
 # Kubernetes namespace to use for all objects created by Metaflow
 KUBERNETES_NAMESPACE = from_conf("METAFLOW_KUBERNETES_NAMESPACE", "default")
-# Service account to use by K8S jobs created by Metaflow
+# Default service account to use by K8S jobs created by Metaflow
 KUBERNETES_SERVICE_ACCOUNT = from_conf("METAFLOW_KUBERNETES_SERVICE_ACCOUNT")
+# Default node selectors to use by K8S jobs created by Metaflow - foo=bar,baz=bab
+KUBERNETES_NODE_SELECTOR = from_conf("METAFLOW_KUBERNETES_NODE_SELECTOR", "")
+KUBERNETES_SECRETS = from_conf("METAFLOW_KUBERNETES_SECRETS", "")
+# Default GPU vendor to use by K8S jobs created by Metaflow (supports nvidia, amd)
+KUBERNETES_GPU_VENDOR = from_conf("METAFLOW_KUBERNETES_GPU_VENDOR", "nvidia")
 # Default container image for K8S
 KUBERNETES_CONTAINER_IMAGE = (
     from_conf("METAFLOW_KUBERNETES_CONTAINER_IMAGE") or DEFAULT_CONTAINER_IMAGE
@@ -197,6 +242,15 @@ KUBERNETES_CONTAINER_REGISTRY = (
 )
 #
 
+##
+# Airflow Configuration
+##
+# This configuration sets `startup_timeout_seconds` in airflow's KubernetesPodOperator.
+AIRFLOW_KUBERNETES_STARTUP_TIMEOUT = from_conf(
+    "METAFLOW_AIRFLOW_KUBERNETES_STARTUP_TIMEOUT_SECONDS", 60 * 60
+)
+
+
 ###
 # Conda configuration
 ###
@@ -206,6 +260,11 @@ CONDA_PACKAGE_S3ROOT = from_conf(
     "%s/conda" % from_conf("METAFLOW_DATASTORE_SYSROOT_S3"),
 )
 
+# Use an alternate dependency resolver for conda packages instead of conda
+# Mamba promises faster package dependency resolution times, which
+# should result in an appreciable speedup in flow environment initialization.
+CONDA_DEPENDENCY_RESOLVER = from_conf("METAFLOW_CONDA_DEPENDENCY_RESOLVER", "conda")
+
 ###
 # Debug configuration
 ###
@@ -240,6 +299,7 @@ if AWS_SANDBOX_ENABLED:
     METADATA_SERVICE_HEADERS["x-api-key"] = AWS_SANDBOX_API_KEY
     SFN_STATE_MACHINE_PREFIX = from_conf("METAFLOW_AWS_SANDBOX_STACK_NAME")
 
+KUBERNETES_SANDBOX_INIT_SCRIPT = from_conf("METAFLOW_KUBERNETES_SANDBOX_INIT_SCRIPT")
 
 # MAX_ATTEMPTS is the maximum number of attempts, including the first
 # task, retries, and the final fallback task and its retries.
@@ -275,24 +335,12 @@ def get_version(pkg):
 # PINNED_CONDA_LIBS are the libraries that metaflow depends on for execution
 # and are needed within a conda environment
 def get_pinned_conda_libs(python_version):
-    if python_version.startswith("3.5"):
-        return {
-            "click": "7.1.2",
-            "requests": "2.24.0",
-            "boto3": "1.9.88",
-            "coverage": "4.5.1",
-        }
-    else:
-        return {
-            "click": "7.1.2",
-            "requests": "2.24.0",
-            "boto3": "1.14.47",
-            "coverage": "4.5.4",
-        }
+    return {
+        "requests": ">=2.21.0",
+        "boto3": ">=1.14.0",
+    }
 
 
-METAFLOW_EXTENSIONS_ADDL_SUFFIXES = set([])
-
 # Check if there are extensions to Metaflow to load and override everything
 try:
     from metaflow.extension_support import get_modules
@@ -308,16 +356,35 @@ try:
                     vars()["METAFLOW_DEBUG_%s" % typ.upper()] = from_conf(
                         "METAFLOW_DEBUG_%s" % typ.upper()
                     )
-            elif n == "METAFLOW_EXTENSIONS_ADDL_SUFFIXES":
-                METAFLOW_EXTENSIONS_ADDL_SUFFIXES.update(o)
+            elif n == "get_pinned_conda_libs":
+
+                def _new_get_pinned_conda_libs(python_version, f1=globals()[n], f2=o):
+                    d1 = f1(python_version)
+                    d2 = f2(python_version)
+                    for k, v in d2.items():
+                        d1[k] = v if k not in d1 else ",".join([d1[k], v])
+                    return d1
+
+                globals()[n] = _new_get_pinned_conda_libs
             elif not n.startswith("__") and not isinstance(o, types.ModuleType):
                 globals()[n] = o
-    METAFLOW_EXTENSIONS_ADDL_SUFFIXES = list(METAFLOW_EXTENSIONS_ADDL_SUFFIXES)
-    if len(METAFLOW_EXTENSIONS_ADDL_SUFFIXES) == 0:
-        METAFLOW_EXTENSIONS_ADDL_SUFFIXES = None
 finally:
     # Erase all temporary names to avoid leaking things
-    for _n in ["m", "n", "o", "ext_modules", "get_modules"]:
+    for _n in [
+        "m",
+        "n",
+        "o",
+        "type",
+        "ext_modules",
+        "get_modules",
+        "_new_get_pinned_conda_libs",
+        "d1",
+        "d2",
+        "k",
+        "v",
+        "f1",
+        "f2",
+    ]:
         try:
             del globals()[_n]
         except KeyError:
