@@ -83,14 +83,14 @@ class MetaflowEnvironment(object):
         cmds = [
             BASH_MFLOG,
             "mflog 'Setting up task environment.'",
-            "%s -m pip install awscli click requests boto3 -qqq" % self._python(),
+            "%s -m pip install awscli requests boto3 -qqq" % self._python(),
             "mkdir metaflow",
             "cd metaflow",
             "mkdir .metaflow",  # mute local datastore creation log
             "i=0; while [ $i -le 5 ]; do "
             "mflog 'Downloading code package...'; "
-            "%s -m awscli s3 cp %s job.tar >/dev/null && \
-                        mflog 'Code package downloaded.' && break; "
+            '%s -m awscli ${METAFLOW_S3_ENDPOINT_URL:+--endpoint-url=\\"${METAFLOW_S3_ENDPOINT_URL}\\"} '
+            "s3 cp %s job.tar >/dev/null && mflog 'Code package downloaded.' && break; "
             "sleep 10; i=$((i+1)); "
             "done" % (self._python(), code_package_url),
             "if [ $i -gt 5 ]; then "
