@@ -18,11 +18,12 @@ from metaflow.metaflow_config import (
     BATCH_METADATA_SERVICE_HEADERS,
     BATCH_EMIT_TAGS,
     DATASTORE_CARD_S3ROOT,
+    S3_ENDPOINT_URL,
 )
 from metaflow.mflog.mflog import refine, set_should_persist
 from metaflow.mflog import (
     export_mflog_env_vars,
-    capture_output_to_mflog,
+    bash_capture_logs,
     tail_logs,
     BASH_SAVE_LOGS,
 )
@@ -61,12 +62,8 @@ class Batch(object):
         )
         init_cmds = environment.get_package_commands(code_package_url)
         init_expr = " && ".join(init_cmds)
-        step_expr = " && ".join(
-            [
-                capture_output_to_mflog(a)
-                for a in (environment.bootstrap_commands(step_name))
-            ]
-            + step_cmds
+        step_expr = bash_capture_logs(
+            " && ".join(environment.bootstrap_commands(step_name) + step_cmds)
         )
 
         # construct an entry point that
@@ -242,13 +239,19 @@ class Batch(object):
         # instance and the remote AWS Batch instance assumes metadata is stored in DATASTORE_LOCAL_DIR
         # on the remote AWS Batch instance; this happens when METAFLOW_DATASTORE_SYSROOT_LOCAL
         # is NOT set (see get_datastore_root_from_config in datastore/local.py).
+        # add METAFLOW_S3_ENDPOINT_URL
+        if S3_ENDPOINT_URL is not None:
+            job.environment_variable("METAFLOW_S3_ENDPOINT_URL", S3_ENDPOINT_URL)
+
         for name, value in env.items():
             job.environment_variable(name, value)
+
         if attrs:
             for key, value in attrs.items():
                 job.parameter(key, value)
         # Tags for AWS Batch job (for say cost attribution)
         if BATCH_EMIT_TAGS:
+            job.tag("app", "metaflow")
             for key in [
                 "metaflow.flow_name",
                 "metaflow.run_id",
@@ -296,7 +299,7 @@ class Batch(object):
                 )
         job = self.create_job(
             step_name,
-            capture_output_to_mflog(step_cli),
+            step_cli,
             task_spec,
             code_package_sha,
             code_package_url,
