@@ -18,6 +18,7 @@ from metaflow.metaflow_config import (
     EVENTS_SFN_ACCESS_IAM_ROLE,
     SFN_DYNAMO_DB_TABLE,
     SFN_EXECUTION_LOG_GROUP_ARN,
+    S3_ENDPOINT_URL,
 )
 from metaflow import R
 
@@ -26,8 +27,6 @@ from .event_bridge_client import EventBridgeClient
 from ..batch.batch import Batch
 from ..aws_utils import compute_resource_attributes
 
-from metaflow.mflog import capture_output_to_mflog
-
 
 class StepFunctionsException(MetaflowException):
     headline = "AWS Step Functions error"
@@ -415,6 +414,10 @@ class StepFunctions(object):
         if env_deco:
             env = env_deco[0].attributes["vars"]
 
+        # add METAFLOW_S3_ENDPOINT_URL
+        if S3_ENDPOINT_URL is not None:
+            env["METAFLOW_S3_ENDPOINT_URL"] = S3_ENDPOINT_URL
+
         if node.name == "start":
             # Initialize parameters for the flow in the `start` step.
             parameters = self._process_parameters()
@@ -725,8 +728,8 @@ class StepFunctions(object):
             "--environment=%s" % self.environment.TYPE,
             "--datastore=%s" % self.flow_datastore.TYPE,
             "--datastore-root=%s" % self.flow_datastore.datastore_root,
-            "--event-logger=%s" % self.event_logger.logger_type,
-            "--monitor=%s" % self.monitor.monitor_type,
+            "--event-logger=%s" % self.event_logger.TYPE,
+            "--monitor=%s" % self.monitor.TYPE,
             "--no-pylint",
             "--with=step_functions_internal",
         ]
@@ -738,14 +741,10 @@ class StepFunctions(object):
             param_file = "".join(
                 random.choice(string.ascii_lowercase) for _ in range(10)
             )
-            export_params = " && ".join(
-                [
-                    capture_output_to_mflog(
-                        "python -m metaflow.plugins.aws.step_functions.set_batch_environment parameters %s"
-                        % param_file
-                    ),
-                    ". `pwd`/%s" % param_file,
-                ]
+            export_params = (
+                "python -m "
+                "metaflow.plugins.aws.step_functions.set_batch_environment "
+                "parameters %s && . `pwd`/%s" % (param_file, param_file)
             )
             params = (
                 entrypoint
@@ -771,7 +770,7 @@ class StepFunctions(object):
             cmd = "if ! %s >/dev/null 2>/dev/null; then %s && %s; fi" % (
                 " ".join(exists),
                 export_params,
-                capture_output_to_mflog(" ".join(params)),
+                " ".join(params),
             )
             cmds.append(cmd)
             paths = "sfn-${METAFLOW_RUN_ID}/_parameters/%s" % (task_id_params)
@@ -780,7 +779,7 @@ class StepFunctions(object):
             parent_tasks_file = "".join(
                 random.choice(string.ascii_lowercase) for _ in range(10)
             )
-            export_parent_tasks = capture_output_to_mflog(
+            export_parent_tasks = (
                 "python -m "
                 "metaflow.plugins.aws.step_functions.set_batch_environment "
                 "parent_tasks %s && . `pwd`/%s" % (parent_tasks_file, parent_tasks_file)
@@ -806,7 +805,7 @@ class StepFunctions(object):
             step.extend("--tag %s" % tag for tag in self.tags)
         if self.namespace is not None:
             step.append("--namespace=%s" % self.namespace)
-        cmds.append(capture_output_to_mflog(" ".join(entrypoint + top_level + step)))
+        cmds.append(" ".join(entrypoint + top_level + step))
         return " && ".join(cmds)
 
 
