@@ -1,15 +1,19 @@
 import base64
-import click
+from metaflow._vendor import click
 from hashlib import sha1
 import json
 import re
 from distutils.version import LooseVersion
 
 from metaflow import current, decorators, parameters, JSONType
-from metaflow.metaflow_config import SFN_STATE_MACHINE_PREFIX
+from metaflow.metaflow_config import (
+    METADATA_SERVICE_VERSION_CHECK,
+    SFN_STATE_MACHINE_PREFIX,
+)
 from metaflow.exception import MetaflowException, MetaflowInternalError
 from metaflow.package import MetaflowPackage
 from metaflow.plugins import BatchDecorator
+from metaflow.tagging_util import validate_tags
 from metaflow.util import get_username, to_bytes, to_unicode
 
 from .step_functions import StepFunctions
@@ -129,11 +133,14 @@ def create(
     workflow_timeout=None,
     log_execution_history=False,
 ):
+    validate_tags(tags)
+
     obj.echo(
         "Deploying *%s* to AWS Step Functions..." % obj.state_machine_name, bold=True
     )
 
-    check_metadata_service_version(obj)
+    if METADATA_SERVICE_VERSION_CHECK:
+        check_metadata_service_version(obj)
 
     token = resolve_token(
         obj.state_machine_name,
