@@ -3,7 +3,7 @@ from metaflow import JSONType, namespace
 from metaflow.exception import CommandException
 import webbrowser
 import re
-import click
+from metaflow._vendor import click
 import os
 import json
 import signal
@@ -171,6 +171,7 @@ def list_available_cards(
     command="view",
     show_list_as_json=False,
     list_many=False,
+    file=None,
 ):
     # pathspec is full pathspec.
     # todo : create nice response messages on the CLI for cards which were found.
@@ -182,7 +183,15 @@ def list_available_cards(
             for tup in path_tuples
         ]
         if not list_many:
-            print(json.dumps(dict(pathspec=pathspec, cards=json_arr), indent=4))
+            # This means that `list_available_cards` is being called once.
+            # So we can directly dump the file
+            dump_dict = dict(pathspec=pathspec, cards=json_arr)
+            if file:
+                with open(file, "w") as f:
+                    json.dump(dump_dict, f)
+            else:
+                ctx.obj.echo_always(json.dumps(dump_dict, indent=4), err=False)
+        # if you have to list many in json format then return
         return dict(pathspec=pathspec, cards=json_arr)
 
     if list_many:
@@ -253,6 +262,7 @@ def list_many_cards(
     card_id=None,
     follow_resumed=None,
     as_json=None,
+    file=None,
 ):
     from metaflow import Flow
 
@@ -288,6 +298,7 @@ def list_many_cards(
                     command=None,
                     show_list_as_json=as_json,
                     list_many=True,
+                    file=file,
                 )
                 if as_json:
                     js_list.append(js_resp)
@@ -299,7 +310,11 @@ def list_many_cards(
             run.pathspec, card_hash=hash, card_type=type, card_id=card_id
         )
     if as_json:
-        print(json.dumps(js_list, indent=4))
+        if file:
+            with open(file, "w") as f:
+                json.dump(js_list, f)
+        else:
+            ctx.obj.echo_always(json.dumps(js_list, indent=4), err=False)
 
 
 @click.group()
@@ -381,7 +396,7 @@ def render_card(mf_card, task, timeout_value=None):
 )
 @click.option(
     "--options",
-    default={},
+    default=None,
     show_default=True,
     type=JSONType,
     help="arguments of the card being created.",
@@ -470,9 +485,12 @@ def create(
         # then check for render_error_card and accordingly
         # store the exception as a string or raise the exception
         try:
-            mf_card = filtered_card(
-                options=options, components=component_arr, graph=graph_dict
-            )
+            if options is not None:
+                mf_card = filtered_card(
+                    options=options, components=component_arr, graph=graph_dict
+                )
+            else:
+                mf_card = filtered_card(components=component_arr, graph=graph_dict)
         except TypeError as e:
             if render_error_card:
                 mf_card = None
@@ -621,6 +639,11 @@ def get(
     is_flag=True,
     help="Print all available cards as a JSON object",
 )
+@click.option(
+    "--file",
+    default=None,
+    help="Save the avaliable card list to file.",
+)
 @click.pass_context
 def list(
     ctx,
@@ -630,7 +653,9 @@ def list(
     id=None,
     follow_resumed=False,
     as_json=False,
+    file=None,
 ):
+
     card_id = id
     if pathspec is None:
         list_many_cards(
@@ -640,6 +665,7 @@ def list(
             card_id=card_id,
             follow_resumed=follow_resumed,
             as_json=as_json,
+            file=file,
         )
         return
 
@@ -659,4 +685,5 @@ def list(
         card_datastore,
         command=None,
         show_list_as_json=as_json,
+        file=file,
     )
