@@ -1,10 +1,16 @@
 import os
+import random
+
 import requests
 import time
 
 from distutils.version import LooseVersion
 
-from metaflow.exception import MetaflowException
+from metaflow.exception import (
+    MetaflowException,
+    MetaflowTaggingError,
+    MetaflowInternalError,
+)
 from metaflow.metaflow_config import (
     METADATA_SERVICE_NUM_RETRIES,
     METADATA_SERVICE_HEADERS,
@@ -12,8 +18,8 @@ from metaflow.metaflow_config import (
 )
 from metaflow.metadata import MetadataProvider
 from metaflow.metadata.heartbeat import HB_URL_KEY
-from metaflow.sidecar import SidecarSubProcess
-from metaflow.sidecar_messages import MessageTypes, Message
+from metaflow.sidecar import Message, MessageTypes, Sidecar
+
 
 # Define message enums
 class HeartbeatTypes(object):
@@ -34,6 +40,7 @@ class ServiceMetadataProvider(MetadataProvider):
     TYPE = "service"
 
     _supports_attempt_gets = None
+    _supports_tag_mutation = None
 
     def __init__(self, environment, flow, event_logger, monitor):
         super(ServiceMetadataProvider, self).__init__(
@@ -46,7 +53,7 @@ class ServiceMetadataProvider(MetadataProvider):
         self.url_run_template = os.path.join(
             METADATA_SERVICE_URL, "flows/{flow_id}/runs/{run_number}/heartbeat"
         )
-        self.sidecar_process = None
+        self.sidecar = None
 
     @classmethod
     def compute_info(cls, val):
@@ -68,19 +75,22 @@ class ServiceMetadataProvider(MetadataProvider):
         return self._version(self._monitor)
 
     def new_run_id(self, tags=None, sys_tags=None):
-        return self._new_run(tags=tags, sys_tags=sys_tags)
+        v, _ = self._new_run(tags=tags, sys_tags=sys_tags)
+        return v
 
     def register_run_id(self, run_id, tags=None, sys_tags=None):
         try:
             # don't try to register an integer ID which was obtained
             # from the metadata service in the first place
             int(run_id)
-            return
+            return False
         except ValueError:
-            return self._new_run(run_id, tags=tags, sys_tags=sys_tags)
+            _, did_create = self._new_run(run_id, tags=tags, sys_tags=sys_tags)
+            return did_create
 
     def new_task_id(self, run_id, step_name, tags=None, sys_tags=None):
-        return self._new_task(run_id, step_name, tags=tags, sys_tags=sys_tags)
+        v, _ = self._new_task(run_id, step_name, tags=tags, sys_tags=sys_tags)
+        return v
 
     def register_task_id(
         self, run_id, step_name, task_id, attempt=0, tags=None, sys_tags=None
@@ -90,11 +100,18 @@ class ServiceMetadataProvider(MetadataProvider):
             # from the metadata service in the first place
             int(task_id)
         except ValueError:
-            self._new_task(
-                run_id, step_name, task_id, attempt, tags=tags, sys_tags=sys_tags
+            _, did_create = self._new_task(
+                run_id,
+                step_name,
+                task_id=task_id,
+                attempt=attempt,
+                tags=tags,
+                sys_tags=sys_tags,
             )
+            return did_create
         else:
-            self._register_code_package_metadata(run_id, step_name, task_id, attempt)
+            self._register_system_metadata(run_id, step_name, task_id, attempt)
+            return False
 
     def _start_heartbeat(
         self, heartbeat_type, flow_id, run_id, step_name=None, task_id=None
@@ -104,15 +121,6 @@ class ServiceMetadataProvider(MetadataProvider):
             # multiple heartbeat side cars of any type/combination. Either a
             # single run heartbeat or a single task heartbeat can be started
             raise Exception("heartbeat already started")
-        # start sidecar
-        if self.version() is None or LooseVersion(self.version()) < LooseVersion(
-            "2.0.4"
-        ):
-            # if old version of the service is running
-            # then avoid running real heartbeat sidecar process
-            self.sidecar_process = SidecarSubProcess("nullSidecarHeartbeat")
-        else:
-            self.sidecar_process = SidecarSubProcess("heartbeat")
         # create init message
         payload = {}
         if heartbeat_type == HeartbeatTypes.TASK:
@@ -132,8 +140,17 @@ class ServiceMetadataProvider(MetadataProvider):
         else:
             raise Exception("invalid heartbeat type")
         payload["service_version"] = self.version()
-        msg = Message(MessageTypes.LOG_EVENT, payload)
-        self.sidecar_process.msg_handler(msg)
+        # start sidecar
+        if self.version() is None or LooseVersion(self.version()) < LooseVersion(
+            "2.0.4"
+        ):
+            # if old version of the service is running
+            # then avoid running real heartbeat sidecar process
+            self.sidecar = Sidecar("none")
+        else:
+            self.sidecar = Sidecar("heartbeat")
+        self.sidecar.start()
+        self.sidecar.send(Message(MessageTypes.BEST_EFFORT, payload))
 
     def start_run_heartbeat(self, flow_id, run_id):
         self._start_heartbeat(HeartbeatTypes.RUN, flow_id, run_id)
@@ -142,11 +159,10 @@ class ServiceMetadataProvider(MetadataProvider):
         self._start_heartbeat(HeartbeatTypes.TASK, flow_id, run_id, step_name, task_id)
 
     def _already_started(self):
-        return self.sidecar_process is not None
+        return self.sidecar is not None
 
     def stop_heartbeat(self):
-        msg = Message(MessageTypes.SHUTDOWN, None)
-        self.sidecar_process.msg_handler(msg)
+        self.sidecar.terminate()
 
     def register_data_artifacts(
         self, run_id, step_name, task_id, attempt_id, artifacts
@@ -158,7 +174,7 @@ class ServiceMetadataProvider(MetadataProvider):
         data = self._artifacts_to_json(
             run_id, step_name, task_id, attempt_id, artifacts
         )
-        self._request(self._monitor, url, data)
+        self._request(self._monitor, url, "POST", data)
 
     def register_metadata(self, run_id, step_name, task_id, metadata):
         url = ServiceMetadataProvider._obj_path(
@@ -166,7 +182,56 @@ class ServiceMetadataProvider(MetadataProvider):
         )
         url += "/metadata"
         data = self._metadata_to_json(run_id, step_name, task_id, metadata)
-        self._request(self._monitor, url, data)
+        self._request(self._monitor, url, "POST", data)
+
+    @classmethod
+    def _mutate_user_tags_for_run(
+        cls, flow_id, run_id, tags_to_add=None, tags_to_remove=None
+    ):
+        min_service_version_with_tag_mutation = "2.3.0"
+        if cls._supports_tag_mutation is None:
+            version = cls._version(None)
+            cls._supports_tag_mutation = version is not None and LooseVersion(
+                version
+            ) >= LooseVersion(min_service_version_with_tag_mutation)
+        if not cls._supports_tag_mutation:
+            raise ServiceException(
+                "Adding or removing tags on a run requires the Metaflow service to be "
+                "at least version %s. Please upgrade your service."
+                % (min_service_version_with_tag_mutation,)
+            )
+
+        url = ServiceMetadataProvider._obj_path(flow_id, run_id) + "/tag/mutate"
+        tag_mutation_data = {
+            # mutate_user_tags_for_run() should have already ensured that this is a list, so let's be tolerant here
+            "tags_to_add": list(tags_to_add or []),
+            "tags_to_remove": list(tags_to_remove or []),
+        }
+        tries = 1
+        status_codes_seen = set()
+        # try up to 10 times, with a gentle exponential backoff (1.4-1.6x)
+        while True:
+            resp, _ = cls._request(
+                None, url, "PATCH", data=tag_mutation_data, return_raw_resp=True
+            )
+            status_codes_seen.add(resp.status_code)
+            # happy path
+            if resp.status_code < 300:
+                return frozenset(resp.json()["tags"])
+            # definitely NOT retriable
+            if resp.status_code in (400, 422):
+                raise MetaflowTaggingError("Metadata service says: %s" % (resp.text,))
+            # if we get here, mutation failure is possibly retriable
+            if tries >= 10:
+                # if we ever received 409 on any of our attempts, report "conflicting updates" blurb to user
+                if 409 in status_codes_seen:
+                    raise MetaflowTaggingError(
+                        "Tagging failed due to too many conflicting updates from other processes"
+                    )
+                # No 409's seen... raise a more generic error
+                raise MetaflowTaggingError("Tagging failed after %d tries" % tries)
+            time.sleep(0.3 * random.uniform(1.4, 1.6) ** tries)
+            tries += 1
 
     @classmethod
     def _get_object_internal(
@@ -194,9 +259,8 @@ class ServiceMetadataProvider(MetadataProvider):
             else:
                 url = ServiceMetadataProvider._obj_path(*args[:obj_order])
             try:
-                return MetadataProvider._apply_filter(
-                    [cls._request(None, url)], filters
-                )[0]
+                v, _ = cls._request(None, url, "GET")
+                return MetadataProvider._apply_filter([v], filters)[0]
             except ServiceException as ex:
                 if ex.http_code == 404:
                     return None
@@ -214,7 +278,8 @@ class ServiceMetadataProvider(MetadataProvider):
         else:
             url += "/%ss" % sub_type
         try:
-            return MetadataProvider._apply_filter(cls._request(None, url), filters)
+            v, _ = cls._request(None, url, "GET")
+            return MetadataProvider._apply_filter(v, filters)
         except ServiceException as ex:
             if ex.http_code == 404:
                 return None
@@ -223,21 +288,22 @@ class ServiceMetadataProvider(MetadataProvider):
     def _new_run(self, run_id=None, tags=None, sys_tags=None):
         # first ensure that the flow exists
         self._get_or_create("flow")
-        run = self._get_or_create("run", run_id, tags=tags, sys_tags=sys_tags)
-        return str(run["run_number"])
+        run, did_create = self._get_or_create(
+            "run", run_id, tags=tags, sys_tags=sys_tags
+        )
+        return str(run["run_number"]), did_create
 
     def _new_task(
         self, run_id, step_name, task_id=None, attempt=0, tags=None, sys_tags=None
     ):
         # first ensure that the step exists
         self._get_or_create("step", run_id, step_name)
-        task = self._get_or_create(
+        task, did_create = self._get_or_create(
             "task", run_id, step_name, task_id, tags=tags, sys_tags=sys_tags
         )
-        self._register_code_package_metadata(
-            run_id, step_name, task["task_id"], attempt
-        )
-        return task["task_id"]
+        if did_create:
+            self._register_system_metadata(run_id, step_name, task["task_id"], attempt)
+        return task["task_id"], did_create
 
     @staticmethod
     def _obj_path(
@@ -297,7 +363,9 @@ class ServiceMetadataProvider(MetadataProvider):
                 self.sticky_tags.union(tags),
                 self.sticky_sys_tags.union(sys_tags),
             )
-            return self._request(self._monitor, create_path, data, obj_path)
+            return self._request(
+                self._monitor, create_path, "POST", data=data, retry_409_path=obj_path
+            )
 
         always_create = False
         obj_path = self._obj_path(self._flow_name, run_id, step_name, task_id)
@@ -311,30 +379,45 @@ class ServiceMetadataProvider(MetadataProvider):
             return create_object()
 
         try:
-            return self._request(self._monitor, obj_path)
+            return self._request(self._monitor, obj_path, "GET")
         except ServiceException as ex:
             if ex.http_code == 404:
                 return create_object()
             else:
                 raise
 
+    # TODO _request() needs a more deliberate refactor at some point, it looks quite overgrown.
     @classmethod
-    def _request(cls, monitor, path, data=None, retry_409_path=None):
+    def _request(
+        cls,
+        monitor,
+        path,
+        method,
+        data=None,
+        retry_409_path=None,
+        return_raw_resp=False,
+    ):
         if cls.INFO is None:
             raise MetaflowException(
                 "Missing Metaflow Service URL. "
                 "Specify with METAFLOW_SERVICE_URL environment variable"
             )
+        supported_methods = ("GET", "PATCH", "POST")
+        if method not in supported_methods:
+            raise MetaflowException(
+                "Only these methods are supported: %s, but got %s"
+                % (supported_methods, method)
+            )
         url = os.path.join(cls.INFO, path.lstrip("/"))
         for i in range(METADATA_SERVICE_NUM_RETRIES):
             try:
-                if data is None:
+                if method == "GET":
                     if monitor:
                         with monitor.measure("metaflow.service_metadata.get"):
                             resp = requests.get(url, headers=METADATA_SERVICE_HEADERS)
                     else:
                         resp = requests.get(url, headers=METADATA_SERVICE_HEADERS)
-                else:
+                elif method == "POST":
                     if monitor:
                         with monitor.measure("metaflow.service_metadata.post"):
                             resp = requests.post(
@@ -344,6 +427,20 @@ class ServiceMetadataProvider(MetadataProvider):
                         resp = requests.post(
                             url, headers=METADATA_SERVICE_HEADERS, json=data
                         )
+                elif method == "PATCH":
+                    if monitor:
+                        with monitor.measure("metaflow.service_metadata.patch"):
+                            resp = requests.patch(
+                                url, headers=METADATA_SERVICE_HEADERS, json=data
+                            )
+                    else:
+                        resp = requests.patch(
+                            url, headers=METADATA_SERVICE_HEADERS, json=data
+                        )
+                else:
+                    raise MetaflowInternalError("Unexpected HTTP method %s" % (method,))
+            except MetaflowInternalError:
+                raise
             except:  # noqa E722
                 if monitor:
                     with monitor.count("metaflow.service_metadata.failed_request"):
@@ -354,8 +451,10 @@ class ServiceMetadataProvider(MetadataProvider):
                         raise
                 resp = None
             else:
+                if return_raw_resp:
+                    return resp, True
                 if resp.status_code < 300:
-                    return resp.json()
+                    return resp.json(), True
                 elif resp.status_code == 409 and data is not None:
                     # a special case: the post fails due to a conflict
                     # this could occur when we missed a success response
@@ -366,9 +465,10 @@ class ServiceMetadataProvider(MetadataProvider):
                     # instead of retrying the post we retry with a get since
                     # the record is guaranteed to exist
                     if retry_409_path:
-                        return cls._request(monitor, retry_409_path)
+                        v, _ = cls._request(monitor, retry_409_path, "GET")
+                        return v, False
                     else:
-                        return
+                        return None, False
                 elif resp.status_code != 503:
                     raise ServiceException(
                         "Metadata request (%s) failed (code %s): %s"
