@@ -14,7 +14,10 @@ from datetime import datetime
 from io import BytesIO
 from functools import partial
 
+from metaflow.datastore.exceptions import DataException
+
 from . import get_namespace
+from .metadata import MetaDatum
 from .metaflow_config import MAX_ATTEMPTS
 from .exception import (
     MetaflowException,
@@ -64,6 +67,8 @@ class NativeRuntime(object):
         monitor,
         run_id=None,
         clone_run_id=None,
+        clone_only=False,
+        reentrant=False,
         clone_steps=None,
         max_workers=MAX_WORKERS,
         max_num_splits=MAX_NUM_SPLITS,
@@ -92,7 +97,9 @@ class NativeRuntime(object):
         self._monitor = monitor
 
         self._clone_run_id = clone_run_id
+        self._clone_only = clone_only
         self._clone_steps = {} if clone_steps is None else clone_steps
+        self._reentrant = reentrant
 
         self._origin_ds_set = None
         if clone_run_id:
@@ -166,6 +173,8 @@ class NativeRuntime(object):
             input_paths=input_paths,
             may_clone=may_clone,
             clone_run_id=self._clone_run_id,
+            clone_only=self._clone_only,
+            reentrant=self._reentrant,
             origin_ds_set=self._origin_ds_set,
             decos=decos,
             logger=self._logger,
@@ -241,9 +250,15 @@ class NativeRuntime(object):
         # assert that end was executed and it was successful
         if ("end", ()) in self._finished:
             self._logger("Done!", system_msg=True)
+        elif self._clone_only:
+            self._logger(
+                "Clone-only resume complete -- only previously successful steps were "
+                "cloned; no new tasks executed!",
+                system_msg=True,
+            )
         else:
             raise MetaflowInternalError(
-                "The *end* step was not successful " "by the end of flow."
+                "The *end* step was not successful by the end of flow."
             )
 
     def _killall(self):
@@ -280,7 +295,7 @@ class NativeRuntime(object):
         self._run_queue.insert(0, (step, task_kwargs))
 
     def _queue_pop(self):
-        return self._run_queue.pop() if self._run_queue else None
+        return self._run_queue.pop() if self._run_queue else (None, {})
 
     def _queue_task_join(self, task, next_steps):
         # if the next step is a join, we need to check that
@@ -539,7 +554,7 @@ class NativeRuntime(object):
                                 or returncode == METAFLOW_EXIT_DISALLOW_RETRY
                             ):
                                 self._logger(
-                                    "This failed task will not be " "retried.",
+                                    "This failed task will not be retried.",
                                     system_msg=True,
                                 )
                             else:
@@ -578,6 +593,15 @@ class NativeRuntime(object):
         self._launch_worker(worker.task)
 
     def _launch_worker(self, task):
+        if self._clone_only and not task.is_cloned:
+            # We don't launch a worker here
+            self._logger(
+                "Not executing non-cloned task for step '%s' in clone-only resume"
+                % "/".join([task.flow_name, task.run_id, task.step]),
+                system_msg=True,
+            )
+            return
+
         worker = Worker(task, self._max_log_size)
         for fd in worker.fds():
             self._workers[fd] = worker
@@ -600,53 +624,31 @@ class Task(object):
         event_logger,
         monitor,
         input_paths=None,
+        may_clone=False,
+        clone_run_id=None,
+        clone_only=False,
+        reentrant=False,
+        origin_ds_set=None,
+        decos=None,
+        logger=None,
+        # Anything below this is passed as part of kwargs
         split_index=None,
         ubf_context=None,
         ubf_iter=None,
-        clone_run_id=None,
-        origin_ds_set=None,
-        may_clone=False,
         join_type=None,
-        logger=None,
         task_id=None,
-        decos=[],
     ):
-        if ubf_context == UBF_CONTROL:
-            [input_path] = input_paths
-            run, input_step, input_task = input_path.split("/")
-            # We associate the control task-id to be 1:1 with the split node
-            # where the unbounded-foreach was defined.
-            # We prefer encoding the corresponding split into the task_id of
-            # the control node; so it has access to this information quite
-            # easily. There is anyway a corresponding int id stored in the
-            # metadata backend - so this should be fine.
-            task_id = "control-%s-%s-%s" % (run, input_step, input_task)
-        # Register only regular Metaflow (non control) tasks.
-        if task_id is None:
-            task_id = str(metadata.new_task_id(run_id, step))
-        else:
-            # task_id is preset only by persist_constants() or control tasks.
-            if ubf_context == UBF_CONTROL:
-                tags = [CONTROL_TASK_TAG]
-                metadata.register_task_id(
-                    run_id,
-                    step,
-                    task_id,
-                    0,
-                    sys_tags=tags,
-                )
-            else:
-                metadata.register_task_id(run_id, step, task_id, 0)
 
         self.step = step
         self.flow_name = flow.name
         self.run_id = run_id
-        self.task_id = task_id
+        self.task_id = None
+        self._path = None
         self.input_paths = input_paths
         self.split_index = split_index
         self.ubf_context = ubf_context
         self.ubf_iter = ubf_iter
-        self.decos = decos
+        self.decos = [] if decos is None else decos
         self.entrypoint = entrypoint
         self.environment = environment
         self.environment_type = self.environment.TYPE
@@ -658,15 +660,14 @@ class Task(object):
         self.monitor = monitor
 
         self._logger = logger
-        self._path = "%s/%s/%s" % (self.run_id, self.step, self.task_id)
 
         self.retries = 0
         self.user_code_retries = 0
         self.error_retries = 0
 
         self.tags = metadata.sticky_tags
-        self.event_logger_type = self.event_logger.logger_type
-        self.monitor_type = monitor.monitor_type
+        self.event_logger_type = self.event_logger.TYPE
+        self.monitor_type = monitor.TYPE
 
         self.metadata_type = metadata.TYPE
         self.datastore_type = flow_datastore.TYPE
@@ -674,10 +675,119 @@ class Task(object):
         self.datastore_sysroot = flow_datastore.datastore_root
         self._results_ds = None
 
+        origin = None
         if clone_run_id and may_clone:
-            self._is_cloned = self._attempt_clone(clone_run_id, join_type)
+            origin = self._find_origin_task(clone_run_id, join_type)
+        if origin and origin["_task_ok"]:
+            # At this point, we know we are going to clone
+            self._is_cloned = True
+            if reentrant:
+                # A re-entrant clone basically allows multiple concurrent processes
+                # to perform the clone at the same time to the same new run id. Let's
+                # assume two processes A and B both simultaneously calling
+                # `resume --reentrant --run-id XX`.
+                # For each task that is cloned, we want to guarantee that:
+                #   - one and only one of A or B will do the actual cloning
+                #   - the other process (or other processes) will block until the cloning
+                #     is complete.
+                # This ensures that the rest of the clone algorithm can proceed as normal
+                # and also guarantees that we only write once to the datastore and
+                # metadata.
+                #
+                # To accomplish this, we use the cloned task's task-id as the "key" to
+                # synchronize on. We then try to "register" this new task-id (or rather
+                # the full pathspec <run>/<step>/<taskid>) with the metadata service
+                # which will indicate if we actually registered it or if it existed
+                # already. If we did manage to register it, we are the "elected cloner"
+                # in essence and proceed to clone. If we didn't, we just wait to make
+                # sure the task is fully done (ie: the clone is finished).
+                if task_id is not None:
+                    # Sanity check -- this should never happen. We cannot allow
+                    # for explicit task-ids because in the reentrant case, we use the
+                    # cloned task's id as the new task's id.
+                    raise MetaflowInternalError(
+                        "Reentrant clone-only resume does not allow for explicit task-id"
+                    )
+
+                # We will use the same task_id as the original task
+                # to use it effectively as a synchronization key
+                clone_task_id = origin.task_id
+                # Make sure the task-id is a non-integer to not clash with task ids
+                # assigned by the metadata provider. If this is an integer, we
+                # add some string to it. It doesn't matter what we add as long as it is
+                # consistent.
+                try:
+                    clone_task_int = int(clone_task_id)
+                    clone_task_id = "resume-%d" % clone_task_int
+                except ValueError:
+                    pass
+
+                # If _get_task_id returns True it means the task already existed so
+                # we wait for it.
+                self._wait_for_clone = self._get_task_id(clone_task_id)
+            else:
+                self._wait_for_clone = False
+                self._get_task_id(task_id)
+
+            # Store the mapping from current_pathspec -> origin_pathspec which
+            # will be useful for looking up origin_ds_set in find_origin_task.
+            self.clone_pathspec_mapping[self._path] = origin.pathspec
+            if self.step == "_parameters":
+                # We don't put _parameters on the queue so we either clone it or wait
+                # for it.
+                if not self._wait_for_clone:
+                    # Clone in place without relying on run_queue.
+                    self.new_attempt()
+                    self._ds.clone(origin)
+                    self._ds.done()
+                else:
+                    # TODO: There is a bit of a duplication with the task.py
+                    # clone_only function here
+                    self.log(
+                        "Waiting for clone of _parameters step to occur...",
+                        system_msg=True,
+                    )
+                    while True:
+                        try:
+                            ds = self._flow_datastore.get_task_datastore(
+                                self.run_id, self.step, self.task_id
+                            )
+                            if not ds["_task_ok"]:
+                                raise MetaflowInternalError(
+                                    "Externally cloned _parameters task did not succeed"
+                                )
+                            break
+                        except DataException:
+                            self.log("Sleeping for 5s...", system_msg=True)
+                            # No need to get fancy with the sleep here.
+                            time.sleep(5)
+                    self.log("_parameters clone successful", system_msg=True)
+            else:
+                # For non parameter steps
+                # Store the origin pathspec in clone_origin so this can be run
+                # as a task by the runtime.
+                self.clone_origin = origin.pathspec
+                # Save a call to creating the results_ds since its same as origin.
+                self._results_ds = origin
+                if self._wait_for_clone:
+                    self.log(
+                        "Waiting for the successful cloning of results "
+                        "of a previously run task %s (this may take some time)"
+                        % self.clone_origin,
+                        system_msg=True,
+                    )
+                else:
+                    self.log(
+                        "Cloning results of a previously run task %s"
+                        % self.clone_origin,
+                        system_msg=True,
+                    )
         else:
             self._is_cloned = False
+            if clone_only:
+                # We are done -- we don't proceed to create new task-ids
+                return
+            self._get_task_id(task_id)
 
         # Open the output datastore only if the task is not being cloned.
         if not self._is_cloned:
@@ -732,6 +842,63 @@ class Task(object):
         self._logger(msg, head=prefix, system_msg=system_msg, timestamp=timestamp)
         sys.stdout.flush()
 
+    def _get_task_id(self, task_id):
+        already_existed = True
+        if self.ubf_context == UBF_CONTROL:
+            [input_path] = self.input_paths
+            run, input_step, input_task = input_path.split("/")
+            # We associate the control task-id to be 1:1 with the split node
+            # where the unbounded-foreach was defined.
+            # We prefer encoding the corresponding split into the task_id of
+            # the control node; so it has access to this information quite
+            # easily. There is anyway a corresponding int id stored in the
+            # metadata backend - so this should be fine.
+            task_id = "control-%s-%s-%s" % (run, input_step, input_task)
+        # Register only regular Metaflow (non control) tasks.
+        if task_id is None:
+            task_id = str(self.metadata.new_task_id(self.run_id, self.step))
+            already_existed = False
+        else:
+            # task_id is preset only by persist_constants() or control tasks.
+            if self.ubf_context == UBF_CONTROL:
+                tags = [CONTROL_TASK_TAG]
+                attempt_id = 0
+                already_existed = not self.metadata.register_task_id(
+                    self.run_id,
+                    self.step,
+                    task_id,
+                    attempt_id,
+                    sys_tags=tags,
+                )
+                # A Task's tags are now those of its ancestral Run so we are not able
+                # to rely on a task's tags to indicate the presence of a control task
+                # so, on top of adding the tags above, we also add a task metadata
+                # entry indicating that this is a "control task".
+                #
+                # Here we will also add a task metadata entry to indicate "control task".
+                # Within the metaflow repo, the only dependency of such a "control task"
+                # indicator is in the integration test suite (see Step.control_tasks() in
+                # client API).
+                task_metadata_list = [
+                    MetaDatum(
+                        field="internal_task_type",
+                        value=CONTROL_TASK_TAG,
+                        type="internal_task_type",
+                        tags=["attempt_id:{0}".format(attempt_id)],
+                    )
+                ]
+                self.metadata.register_metadata(
+                    self.run_id, self.step, task_id, task_metadata_list
+                )
+            else:
+                already_existed = not self.metadata.register_task_id(
+                    self.run_id, self.step, task_id, 0
+                )
+
+        self.task_id = task_id
+        self._path = "%s/%s/%s" % (self.run_id, self.step, self.task_id)
+        return already_existed
+
     def _find_origin_task(self, clone_run_id, join_type):
         if self.step == "_parameters":
             pathspec = "%s/_parameters[]" % clone_run_id
@@ -769,32 +936,6 @@ class Task(object):
             pathspec = "%s/%s[%s]" % (clone_run_id, self.step, index)
             return self.origin_ds_set.get_with_pathspec_index(pathspec)
 
-    def _attempt_clone(self, clone_run_id, join_type):
-        origin = self._find_origin_task(clone_run_id, join_type)
-
-        if origin and origin["_task_ok"]:
-            # Store the mapping from current_pathspec -> origin_pathspec which
-            # will be useful for looking up origin_ds_set in find_origin_task.
-            self.clone_pathspec_mapping[self._path] = origin.pathspec
-            if self.step == "_parameters":
-                # Clone in place without relying on run_queue.
-                self.new_attempt()
-                self._ds.clone(origin)
-                self._ds.done()
-            else:
-                self.log(
-                    "Cloning results of a previously run task %s" % origin.pathspec,
-                    system_msg=True,
-                )
-                # Store the origin pathspec in clone_origin so this can be run
-                # as a task by the runtime.
-                self.clone_origin = origin.pathspec
-                # Save a call to creating the results_ds since its same as origin.
-                self._results_ds = origin
-            return True
-        else:
-            return False
-
     @property
     def path(self):
         return self._path
@@ -818,6 +959,10 @@ class Task(object):
     def is_cloned(self):
         return self._is_cloned
 
+    @property
+    def wait_for_clone(self):
+        return self._wait_for_clone
+
     def persist(self, flow):
         # this is used to persist parameters before the start step
         flow._task_ok = flow._success = True
@@ -887,10 +1032,10 @@ class CLIArgs(object):
         self.entrypoint = list(task.entrypoint)
         self.top_level_options = {
             "quiet": True,
-            "coverage": "coverage" in sys.modules,
             "metadata": self.task.metadata_type,
             "environment": self.task.environment_type,
             "datastore": self.task.datastore_type,
+            "pylint": False,
             "event-logger": self.task.event_logger_type,
             "monitor": self.task.monitor_type,
             "datastore-root": self.task.datastore_sysroot,
@@ -1001,8 +1146,11 @@ class Worker(object):
 
         if self.task.is_cloned and self.task.clone_origin:
             args.command_options["clone-only"] = self.task.clone_origin
-            # disabling atlas sidecar for cloned tasks due to perf reasons
+            # disabling sidecars for cloned tasks due to perf reasons
+            args.top_level_options["event-logger"] = "nullSidecarLogger"
             args.top_level_options["monitor"] = "nullSidecarMonitor"
+            if self.task.wait_for_clone:
+                args.command_options["clone-wait-only"] = True
         else:
             # decorators may modify the CLIArgs object in-place
             for deco in self.task.decos:
